/* usbn960x.c
 * Copyright (C) 2006  Benedikt Sauter
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <avr/pgmspace.h>
#include <stdlib.h>
#include "usbn960x.h"
#include "../../usbn2mc.h"

static EPInfo EP0rx;
static EPInfo EP0tx;

uint8_t PROGMEM _USBN_langDesc[] = { 0x04, STRING, 0x09, 0x04 };
USB_StringDescriptor_t StringDescriptors[10];
// FunctionInfo  USBNFunctionInfo;

void _USBNReceiveFIFO0(void);
void _USBNTransmitFIFO0(void);

void _USBNInitEP0(void) {
	EP0rx.usbnCommand = RXC0;
	EP0rx.usbnData    = RXD0;
	EP0rx.usbnControl = EPC0;
	EP0rx.DataPid     = 0;
	EP0rx.usbnfifo    = 8;
	EP0rx.bufType     = EP_MEM;

	EP0tx.usbnCommand = TXC0;
	EP0tx.usbnData    = TXD0;
	EP0tx.usbnControl = EPC0;
	EP0tx.DataPid     = 0;
	EP0tx.usbnfifo    = 8;
	EP0tx.bufType     = EP_MEM;
}


// ********************************************************************
// Interrupt Event Handler
// ********************************************************************

void _USBNNackEvent(void) {
	uint8_t event;
	event = USBNRead(NAKEV);
	//USBNWrite(RXC1,FLUSH);	//re-enable the receiver
	//USBNWrite(RXC1,RX_EN);	//re-enable the receiver
	/*
	 if (EP0tx.Size > EP0tx.usbnfifo)	  //multi-pkt status stage?
	 {
	 //USBNDebug("flush");
	 USBNWrite(TXC0,FLUSH);	    //flush TX0 and disable
	 //USBNWrite(RXC0,RX_EN);	//re-enable the receiver
	 EP0tx.DataPid	      = 1;
	 }
	 */
}

void _USBNReceiveEvent(void) {
//	char buf[64];
//	char *bufp = &buf[0];
	uint8_t event = USBNRead(RXEV);

	if (event & RX_FIFO0)
		_USBNReceiveFIFO0();
	else if (event & RX_FIFO1 && RX1Callback != 0) // dynamic function call
		RX1Callback(USBNRead(RXS1)&0x0f);
}

void _USBNTransmitEvent(void) {
	uint8_t event;
	event = USBNRead(TXEV);
	//USBNDebug("tx event\r\n");
	if (event & TX_FIFO0)
		_USBNTransmitFIFO0();
	else {
		USBNDebug("tx event\r\n");
		USBNRead(TXS1); // get transmitter status
		USBNRead(TXS2); // get transmitter status
		USBNRead(TXS3); // get transmitter status
	}
}

void _USBNAlternateEvent(void) {
	uint8_t event;
	event = USBNRead(ALTEV);
	USBNDebug("alt event\r\n");

	if (event & ALT_RESET) {
		USBNWrite(NFSR, RST_ST); // NFS = NodeReset
		USBNWrite(FAR, AD_EN + 0);
		USBNWrite(EPC0, 0x00);
		USBNWrite(TXC0, FLUSH);
		USBNWrite(RXC0, RX_EN); // allow reception
		USBNWrite(NFSR, OPR_ST); // NFS = NodeOperational
		USBNDebug("reset\r\n");
	}
	if (event & ALT_SD3) {
		USBNWrite(ALTMSK, ALT_RESUME + ALT_RESET); // adjust interrupts
		/*
		 USBNWrite(NFSR,SUS_ST);                   // enter suspend state
		 USBNDebug("sd3\r\n");
		 */
	}
	if (event & ALT_RESUME) {
		USBNWrite(ALTMSK, ALT_SD3 + ALT_RESET + ALT_RESUME);
		/*
		 USBNWrite(EPC0,0x00);
		 USBNWrite(RXC0,RX_EN);                    // allow reception
		 USBNWrite(TXC0,FLUSH);
		 USBNWrite(NFSR,OPR_ST);
		 USBNDebug("resume\r\n");
		 */
	}
	if (event & ALT_EOP) {
		USBNDebug("eop\r\n");
	}

}


// ********************************************************************
// Receive and Transmit functions for EPs
// ********************************************************************

void _USBNReceiveFIFO0(void) {
	uint8_t rxstatus;
	char Buf[8];
	DeviceRequest *req;
	int i;

	USBNDebug("rx\r\n");

	EP0tx.bufType = EP_MEM;
	rxstatus = USBNRead(RXS0);

	if (rxstatus & SETUP_R) {
		for (i = 0; i < 8; i++) {
			Buf[i] = USBNRead(EP0rx.usbnData);
		}

#if DEBUG
		for(i=0;i<8;i++)
		SendHex(Buf[i]); // type - get descr or set address
		USBNDebug("\r\n");
#endif

		req = (DeviceRequest*) (Buf);

		USBNWrite(RXC0, FLUSH); // make sure the RX is off
		USBNWrite(TXC0, FLUSH); // make sure the TX is off
		USBNWrite(EPC0, USBNRead(EPC0) & 0x7F); // turn of stall
		// noch ein switch um zu entscheiden obs fuers device, interface endpoint oder andere ist

		switch (req->bmRequestType & 0x60) // decode request type
		{
		case DO_STANDARD: // standard request
			if ((req->bmRequestType & 0x1f) == 0x00) {
				switch (req->bRequest) // decode request code
				{
#if 0
				case CLR_FEATURE:
				USBNDebug("CLR FEATURE\n\r");
				//_USBNClearFeature(req);
				break;
				case GET_CONFIGURATION:
				//USBNDebug("GET CONFIG\n\r");
				USBNWrite(TXD0,USBNFunctionInfo.ConfigurationIndex);
				break;
#endif
				case GET_DESCRIPTOR:
					USBNDebug("GET DESCRIPTOR\n\r");
					_USBNGetDescriptor(req);
					break;
				case GET_INTERFACE:
					USBNDebug("GET INTERFACE\n\r");
					break;
				case GET_STATUS:
					USBNDebug("GET STATUS\n\r");
					USBNWrite(TXC0,FLUSH);
					USBNWrite(TXD0,1);
					USBNWrite(TXD0,0);
					USBNWrite(TXC0,TX_TOGL+TX_EN);  //enable the TX (DATA1)
					break;
				case SET_ADDRESS:
					USBNDebug("SET ADDRESS ");
					USBNWrite(EPC0, DEF);
					USBNWrite(FAR, AD_EN + req->wValue);
					break;
				case SET_CONFIGURATION:
					USBNDebug("SET CONFIGURATION\n\r");
					_USBNSetConfiguration(req);
					break;
#if 0
					case SET_FEATURE:
					USBNDebug("SET FEATURE\n\r");
					break;
#endif
				case SET_INTERFACE:
					USBNDebug("SET INTERFACE\n\r");
					//if(EP0rx.Buf[2])
					//   USBNWrite(EPC1,0);      // stall the endpoint
					USBNWrite(TXC0, TX_TOGL + TX_EN); //enable the TX (DATA1)
					break;
				default: // unsupported standard req
					//#if DEBUG
					USBNDebug("unsupported standard req\n\r");
					//#endif
					USBNWrite(EPC0, USBNRead(EPC0) + STALL); // stall the endpoint
					break;
				}
			} else {
				// default request but for interface not for device
				USBNInterfaceRequests(req, &EP0tx);
				_USBNTransmit(&EP0tx);
			}
			break;
			//#if 0
		case DO_CLASS: // class request
			USBNDebug("Class request\n\r");
			USBNDecodeClassRequest(req, &EP0tx);
			_USBNTransmit(&EP0tx);
			//USBNWrite(TXC0,TX_TOGL+TX_EN);  //enable the TX (DATA1)
			break;
		case DO_VENDOR: // vendor request
			USBNDebug("Vendor request\n\r");
			USBNDecodeVendorRequest(req);
			_USBNTransmit(&EP0tx);
			//USBNWrite(TXC0,TX_TOGL+TX_EN);  //enable the TX (DATA1)
			break;
		default: // unsupported req type
			USBNDebug("unsupported req type\r\n");
			USBNWrite(EPC0, USBNRead(EPC0) + STALL); // stall the endpoint
			break;
		}
		//#endif
		//the following is done for all setup packets.  Note that if
		//no data was stuffed into the FIFO, the result of the fol-
		//lowing will be a zero-length response.

		// only for stage 2 transfers
		if (req->bmRequestType == 0x00)
			USBNWrite(TXC0, TX_TOGL + TX_EN); //enable the TX (DATA1)
	} else // if not a setuppacket
	{
		//USBNDebug("error transmit\r\n");
		if (EP0tx.Size > EP0tx.usbnfifo) // multi-pkt status stage?
		{
#if 0
			if ((rxstatus& 0x5F)!=0x10) // length error??
			{
				USBNDebug("length error\r\n");
			}
#endif
			EP0tx.Size = 0; // exit multi-packet mode
			USBNWrite(TXC0, FLUSH); // flush TX0 and disable
			USBNWrite(RXC0, RX_EN); // re-enable the receiver
		}
	}
}

void _USBNTransmitFIFO0(void) {
	uint8_t txstat;

	txstat = USBNRead(TXS0); // get transmitter status
	USBNDebug("t");

	if (txstat & TX_DONE) // if transmit completed
	{
		USBNWrite(TXC0, FLUSH); // flush TX0 and disable

		if (txstat & ACK_STAT) // ACK received
		{
			if (EP0tx.Index < EP0tx.Size) {
				_USBNTransmit(&EP0tx);
			} else // not in multi-packet mode
			{
				USBNWrite(RXC0, RX_EN); // re-enable the receiver
			}
		} else
		// this probably means we issued a stall handshake
		{
			//USBNDebug("stall handshake ");
			USBNWrite(RXC0, RX_EN); // re-enable the receiver
		}
	}
	// otherwise something must have gone wrong with the previous
	// transmission, or we got here somehow we shouldn't have
	else {
		//USBNDebug("tx0 error\n");
	}
	// we do this stuff for all tx_0 events
}

// ********************************************************************
// Communicate with the FIFOs
// ********************************************************************

// toggle mechanism
inline void _USBNToggle(EPInfo* ep) {
	uint8_t togl = 0;
	if(ep->DataPid != 0x00) togl = TX_TOGL;
	USBNWrite(ep->usbnCommand, togl + TX_EN);
	ep->DataPid ^= 0x01;
}

/**
 * Transmit data from RAM to endpoint.
 */
inline void _USBNDataToFifo(EPInfo* ep) {
	uint8_t i;
	for (i=0; (i < ep->usbnfifo) && (ep->Index < ep->Size); i++) {
		USBNWrite(ep->usbnData, ep->Buf[ep->Index++]);
	}
}

/**
 * Transmit data from flash memory to endpoint.
 */
inline void _USBNDataToFifo_P(EPInfo* ep) {
	uint8_t i;
	for (i=0; (i < ep->usbnfifo) && (ep->Index < ep->Size); i++) {
		USBNWrite(ep->usbnData, pgm_read_byte(ep->Buf + ep->Index));
		ep->Index++;
	}
}

/**
 * Transmit a string descriptor from flash memory to endpoint.
 */
inline void _USBNStringToFifo_P(EPInfo* ep) {
	uint8_t i=0;

	if(ep->Index == 0) {							// first send descriptor header
		USBNWrite(ep->usbnData, ep->Size*2 + 2);	//    length will be double the string length
		USBNWrite(ep->usbnData, STRING);			//    descriptor type
		i=2;
	}
	for (; (i < ep->usbnfifo) && (ep->Index < ep->Size); i+=2) {
		USBNWrite(ep->usbnData, pgm_read_byte(ep->Buf + ep->Index++));		// transmit character
		USBNWrite(ep->usbnData, 0x00);										// fill in zero
	}
}

void _USBNReceive(EPInfo* ep) {
	uint8_t count = ep->Size;
	for (ep->Index = 0; ep->Index < count; ep->Index++) {
		ep->Buf[ep->Index] = USBNRead(ep->usbnData);
	}
	USBNWrite(ep->usbnCommand, FLUSH);
	USBNWrite(ep->usbnCommand, RX_EN);
}

void _USBNTransmit(EPInfo* ep) {
//	uint8_t i;
	if (ep->Size == 0 || ep->Buf == 0) return;	// data in buffer?

	if (ep->Index < ep->Size) {			// more data to send?

		USBNWrite(ep->usbnCommand, FLUSH); //send data to the FIFO
		if (ep->bufType == EP_MEM)
			_USBNDataToFifo(ep);
		else if (ep->bufType == EP_FLASH)
			_USBNDataToFifo_P(ep);
		else if (ep->bufType == EP_STRING)
			_USBNStringToFifo_P(ep);

	}

	if (ep->Index >= ep->Size && ep == &EP0tx)	// was this the last packet?
		USBNWrite(RXC0, RX_EN);

	_USBNToggle(ep);
}


// ********************************************************************
// standard request from EP0
// ********************************************************************

void _USBNGetDescriptor(DeviceRequest *req) {
	uint8_t index = req->wValue;
	uint8_t type = req->wValue >> 8;

	EP0tx.Index = 0;
	EP0tx.DataPid = 1;
	switch (type) {
	case DEVICE:
		USBNDebug("DEVICE DESCRIPTOR\n\r");
		EP0tx.bufType = EP_FLASH;
		EP0tx.Size = pgm_read_byte(&DeviceDescriptor[0]);
		EP0tx.Buf = DeviceDescriptor;

		// first get descriptor request is
		// always be answered with first 8 uint8_ts of dev descriptor
		if (req->wLength == 0x40)
			EP0tx.Size = 8;
		break;
	case CONFIGURATION:
		USBNDebug("CONFIGURATION DESCRIPTOR ");

		// send complete tree
		EP0tx.bufType = EP_FLASH;
		EP0tx.Size = req->wLength;
		EP0tx.Buf = ConfigurationDescriptor;

		break;
	case STRING:
		if (index > 0) {
			index--;
			EP0tx.bufType = EP_STRING;
			EP0tx.Buf = StringDescriptors[index].String;
			EP0tx.Size = StringDescriptors[index].bLength;
		} else {
			EP0tx.bufType = EP_FLASH;
			EP0tx.Buf = _USBN_langDesc;
			EP0tx.Size = 4;
		}
		break;
	}

	_USBNTransmit(&EP0tx);
}
