/* usbn960x.c
* Copyright (C) 2005  Benedikt Sauter
* Copyright (C) 2008  Manuel Stahl
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _USBN960X_H
#define _USBN960X_H

#define DEBUG 0

#include <stdint.h>
#include "../usbn960xreg.h"
#include "../usb11spec.h"

/*-------------------------------------------
 * type definitions
 * ------------------------------------------*/

/**
 * Callback when data was received.
 * @param count	number of bytes received
 */
typedef void (FIFOCallback)(uint8_t count);

// TODO: external?
uint8_t *DeviceDescriptor;
uint8_t *ConfigurationDescriptor;
uint8_t USB_numStringDescriptors;
extern USB_StringDescriptor_t StringDescriptors[];

//struct string_entry*  StringList;
//char** FinalStringArray;



FIFOCallback* RX1Callback;


/*-------------------------------------------
 * global data structs
 * ------------------------------------------*/

typedef struct functioninfo FunctionInfo;
struct functioninfo {
	uint8_t Address;
	uint8_t ConfigurationIndex;
};


typedef struct devicereq DeviceRequest;
struct devicereq {
	uint8_t  bmRequestType;
	uint8_t  bRequest;
	uint16_t wValue;
	uint16_t wIndex;
	uint16_t wLength;
};

typedef enum {
	EP_MEM,
	EP_FLASH,
	EP_STRING
} BufferType_t;

typedef struct epinfo EPInfo;
struct epinfo {
	uint8_t usbnData;
	uint8_t usbnCommand;
	uint8_t usbnControl;
	uint8_t DataPid;		// 0 = data0, 1 = data1
	BufferType_t bufType;	// location of buffer
	uint8_t usbnfifo;
	uint8_t Index;
	uint8_t Size;
	uint8_t* Buf;
};

uint8_t EP0RXBuf[8];


/*-------------------------------------------
 * system functions
 * ------------------------------------------*/

void _USBNReceiveEvent(void);
void _USBNTransmitEvent(void);
void _USBNNackEvent(void);
void _USBNAlternateEvent(void);


/// usb default requests set address
void _USBNGetDescriptor(DeviceRequest *req);
void _USBNSetConfiguration(DeviceRequest *req);
void _USBNClearFeature(void);

void _USBNToggle(EPInfo* ep);
void _USBNTransmit(EPInfo* ep);
void _USBNReceive(EPInfo* ep);

/**
 * Read a byte from the USBN controller.
 * Must be implemented for the used microcontroller and wiring.
 * @param Adr	the address to read from
 * @return	the byte that was read
 */
uint8_t USBNRead(uint8_t Adr);
uint8_t USBNBurstRead(void);

/**
 * Write a byte to the USBN controller.
 * Must be implemented for the used microcontroller and wiring.
 * @param Adr	the address to write to
 * @param Data	the byte to write
 */
void USBNWrite(uint8_t Adr,uint8_t Data);
inline void USBNBurstWrite(uint8_t Data);


void _USBNInitEP0(void);

void USBNDebug(char *msg);

/**
 * Decode a vender request.
 * Must be implemented depending on the application.
 * @param req	the request to decode
 */
void USBNDecodeVendorRequest(DeviceRequest *req);
/**
 * Decode a class request.
 * Must be implemented depending on the USB device class.
 * @param req	the request to decode
 * @param ep	TODO: for what do we need the ep?
 */
void USBNDecodeClassRequest(DeviceRequest *req,EPInfo* ep);

#endif /* __USBN960X_H__ */
