/* usb11spec.h
* Copyright (C) 2005  Benedikt Sauter
* Copyright (C) 2008  Manuel Stahl
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _USB11PSEC_H
#define _USB11SPEC_H


/*-------------------------------------------
 * device descriptor
 * ------------------------------------------*/
struct usb_device_descriptor {
  uint8_t   bLength;
  uint8_t   bDescriptorType;
  uint16_t  bcdUSB;
  uint8_t   bDeviceClass;
  uint8_t   bDeviceSubClass;
  uint8_t   bDeviceProtocol;
  uint8_t   bMaxPacketSize0;
  uint16_t  idVendor;
  uint16_t  idProduct;
  uint16_t  bcdDevice;
  uint8_t   iManufacturer;
  uint8_t   iProduct;
  uint8_t   iSerialNumber;
  uint8_t   bNumConfigurations;
};
typedef struct usb_device_descriptor USB_DeviceDescriptor_t;

/*-------------------------------------------
 * configuration descriptor
 * ------------------------------------------*/
struct usb_configuration_descriptor {
  uint8_t   bLength;
  uint8_t   bDescriptorType;
  uint16_t  wTotalLength;
  uint8_t   bNumInterfaces;
  uint8_t   bConfigurationValue;
  uint8_t   iConfiguration;
  uint8_t   bmAttributes;
  uint8_t   MaxPower;
};
typedef struct usb_configuration_descriptor USB_ConfigDescriptor_t;

/*-------------------------------------------
 * interface descriptor
 * ------------------------------------------*/
struct usb_interface_descriptor {
  uint8_t   bLength;
  uint8_t   bDescriptorType;
  uint8_t   bInterfaceNumber;
  uint8_t   bAlternateSetting;
  uint8_t   bNumEndpoints;
  uint8_t   bInterfaceClass;
  uint8_t   bInterfaceSubClass;
  uint8_t   bInterfaceProtocol;
  uint8_t   iInterface;
};
typedef struct usb_interface_descriptor USB_InterfaceDescriptor_t;

/*-------------------------------------------
 * endpoint descriptor
 * ------------------------------------------*/
struct usb_endpoint_descriptor {
  uint8_t   bLength;
  uint8_t   bDescriptorType;
  uint8_t   bEndpointAddress;
  uint8_t   bmAttributes;
  uint16_t  wMaxPacketSize;
  uint8_t   bIntervall;
};
typedef struct usb_endpoint_descriptor USB_EndpointDescriptor_t;

/*-------------------------------------------
 * string descriptor
 * ------------------------------------------*/
struct usb_string_descriptor {
  uint8_t   bLength;
  uint8_t   bDescriptorType;
  uint8_t*  String;
};
typedef struct usb_string_descriptor USB_StringDescriptor_t;


/*------------------------------------------
 * transfer types
 * ------------------------------------------*/
#define CTRL		0x00
#define ISOC		0x01
#define BULK		0x02
#define INTR		0x03

/*-------------------------------------------
 * device descriptor types
 * ------------------------------------------*/
#define DO_STANDARD		0x00
#define DO_CLASS		0x20
#define DO_VENDOR		0x40


/*-------------------------------------------
 * standard requests for ep0
 * ------------------------------------------*/
#define GET_STATUS	  		0x00
#define CLR_FEATURE	  		0x01
#define SET_FEATURE	  		0x03
#define SET_ADDRESS	  		0x05
#define GET_DESCRIPTOR	  	0x06
#define SET_DESCRIPTOR	  	0x07
#define GET_CONFIGURATION 	0x08
#define SET_CONFIGURATION 	0x09
#define GET_INTERFACE	  	0x0A
#define SET_INTERFACE	  	0x0B


/*-------------------------------------------
 * descriptor types
 * ------------------------------------------*/
#define DEVICE					0x01
#define CONFIGURATION			0x02
#define STRING					0x03
#define INTERFACE				0x04
#define ENDPOINT				0x05
#define DEVICEQUALIFIER			0x06 // only usb2.0
#define OTHERSPEEDCONFIGURATION 0x07 // only usb2.0

/*-------------------------------------------
 * class codes
 * ------------------------------------------*/
#define CLS_DEVICE				0x00
#define CLS_AUDIO				0x01
#define CLS_CDC					0x02
#define CLS_HID					0x03
#define CLS_PHYSICAL			0x05
#define CLS_IMAGE				0x06
#define CLS_PRINTER				0x07
#define CLS_MASS_STORAGE		0x08
#define CLS_HUB					0x09
#define CLS_CDC_DATA			0x0A
#define CLS_SMART_CARD			0x0B
#define CLS_CONTENT_SECURITY	0x0D
#define CLS_VIDEO				0x0E
#define CLS_PERS_HEALTH			0x0F
#define CLS_DIAG_DEVICE			0xDC
#define CLS_WIRELESS_CTRL		0xE0
#define CLS_MISC				0xEF
#define CLS_APP_SPECIFIC		0xFE
#define CLS_VENDOR_SPECIFIC		0xFF

#endif /* __USB11SPEC_H__ */
